# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.bindings.stix_common as stix_common_binding
import stix.bindings.stix_core as stix_core_binding
from stix.common import InformationSource, StructuredText, VocabString
from stix.data_marking import Marking

from stix.common.vocabs import PackageIntent

class STIXHeader(stix.Entity):
    _binding = stix_core_binding
    _namespace = 'http://stix.mitre.org/stix-1'

    def __init__(self, package_intents=None, description=None, handling=None,
        information_source=None, title=None, short_description=None):

        self.package_intents = package_intents
        self.title = title
        self.description = description
        self.short_description=short_description
        self.handling = handling
        self.information_source = information_source
        self.profiles = []

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        '''Sets the value of the description property.

        If the value is an instance of basestring, it will be coerced into an
        instance of StructuredText, with its 'text' property set to the input
        value.
        '''

        if value and isinstance(value, basestring):
            st = StructuredText()
            st.value = value
            self._description = st
        elif isinstance(value, StructuredText):
            self._description = value
        elif not value:
            self._description = None
        else:
            raise ValueError('value must be instance of StructuredText or basestring')

    @property
    def short_description(self):
        """The ``short_description`` property for this entity.

        Default Value: ``None``

        Note:
            If set to a value that is not an instance of
            :class:`stix.common.structured_text.StructuredText`, an attempt to
            will be made to convert the value into an instance of
            :class:`stix.common.structured_text.StructuredText`.

        Returns:
            An instance of
            :class:`stix.common.structured_text.StructuredText`

        """
        return self._short_description

    @short_description.setter
    def short_description(self, value):
        if value:
            if isinstance(value, StructuredText):
                self._short_description = value
            else:
                self._short_description = StructuredText(value=value)
        else:
            self._short_description = None


    @property
    def handling(self):
        return self._handling

    @handling.setter
    def handling(self, value):
        if value and not isinstance(value, Marking):
            raise ValueError('value must be instance of Marking')

        self._handling = value

    @property
    def package_intents(self):
        return self._package_intents

    @package_intents.setter
    def package_intents(self, value):
        self._package_intents = PackageIntents()
        if not value:
            return
        elif isinstance(value, list):
            for v in value:
                self.add_package_intent(v)
        else:
            self.add_package_intent(value)

    def add_package_intent(self, package_intent):
        if not package_intent:
            return
        elif isinstance(package_intent, VocabString):
            self.package_intents.append(package_intent)
        else:
            tmp_package_intent = PackageIntent(value=package_intent)
            self.package_intents.append(tmp_package_intent)

    @property
    def information_source(self):
        return self._information_source

    @information_source.setter
    def information_source(self, value):
        if value and not isinstance(value, InformationSource):
            raise ValueError('value must instance of InformationSource')

        self._information_source = value

    def add_profile(self, profile):
        """Adds a profile to the STIX Header. A Profile is represented by a string URI"""
        self.profiles.append(profile)

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.title = obj.Title
        return_obj.description = StructuredText.from_obj(obj.Description)
        return_obj.short_description = StructuredText.from_obj(obj.Short_Description)
        return_obj.handling = Marking.from_obj(obj.Handling)
        return_obj.information_source = InformationSource.from_obj(obj.Information_Source)

        if obj.Package_Intent:
            return_obj.package_intents = [VocabString.from_obj(x) for x in obj.Package_Intent]
        if obj.Profiles:
            return_obj.profiles = obj.Profiles.Profile

        return return_obj

    def to_obj(self, return_obj=None, ns_info=None):
        super(STIXHeader, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding.STIXHeaderType()

        if self.title:
            return_obj.Title = self.title
        if self.package_intents:
            return_obj.Package_Intent = [x.to_obj(ns_info=ns_info) for x in self.package_intents]
        if self.description:
            return_obj.Description = self.description.to_obj(ns_info=ns_info)
        if self.short_description:
            return_obj.Short_Description = self.short_description.to_obj(ns_info=ns_info)
        if self.handling:
            return_obj.Handling = self.handling.to_obj(ns_info=ns_info)
        if self.information_source:
            return_obj.Information_Source = self.information_source.to_obj(ns_info=ns_info)
        if self.profiles:
            profiles_obj = stix_common_binding.ProfilesType(Profile=self.profiles)
            return_obj.Profiles = profiles_obj

        return return_obj

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.title = dict_repr.get('title')
        return_obj.package_intents = [VocabString.from_dict(x) for x in dict_repr.get('package_intents', [])]
        return_obj.description = StructuredText.from_dict(dict_repr.get('description'))
        return_obj.short_description = StructuredText.from_dict(dict_repr.get('short_description'))
        return_obj.handling = Marking.from_dict(dict_repr.get('handling'))
        return_obj.information_source = InformationSource.from_dict(dict_repr.get('information_source'))
        return_obj.profiles = dict_repr.get('profiles')

        return return_obj

    def to_dict(self):
        d = {}
        if self.title:
            d['title'] = self.title
        if self.package_intents:
            d['package_intents'] = [x.to_dict() for x in self.package_intents]
        if self.description:
            d['description'] = self.description.to_dict()
        if self.short_description:
            d['short_description'] = self.short_description.to_dict()
        if self.handling:
            d['handling'] = self.handling.to_dict()
        if self.information_source:
            d['information_source'] = self.information_source.to_dict()
        if self.profiles:
            d['profiles'] = self.profiles

        return d


class PackageIntents(stix.EntityList):
    _namespace = "http://stix.mitre.org/stix-1"
    _contained_type = VocabString

    def _fix_value(self, value):
        return PackageIntent(value)



import unittest

from stix.test import EntityTestCase

from stix.data_marking import Marking, MarkingSpecification, MarkingStructure
import stix.extensions.marking.tlp


class MarkingTests(EntityTestCase, unittest.TestCase):
    klass = Marking
    _full_dict = [
        {
            'id': "foo",
        },
        {
            'id': "bar",
        },
    ]


class MarkingSpecificationTests(EntityTestCase, unittest.TestCase):
    klass = MarkingSpecification
    _full_dict = {
        'id': "foo",
        'idref': "foo_ref",
        'version': "1234",
        'controlled_structure': "some xpath",
        'marking_structures': [
            {
                'marking_model_name': 'TLP',
                'xsi:type': "tlpMarking:TLPMarkingStructureType",
            }
        ]
    }


class MarkingStructureTests(unittest.TestCase):

    def test_xsi_type_required(self):
        d = {
            'marking_model_name': 'TLP',
        }

        # If there's not an xsi:type in the dict, this will raise an error.
        self.assertRaises(ValueError, MarkingStructure.from_dict, d)

    def test_xsi_type_required(self):
        d = {
            'marking_model_name': 'TLP',
            'xsi:type': "UNKNOWN_XSI_TYPE",
        }

        self.assertRaises(ValueError, MarkingStructure.from_dict, d)



if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.bindings.threat_actor as threat_actor_binding
from stix.common import (Confidence, Identity, InformationSource, Statement,
        StructuredText, VocabString)
from stix.common.related import (GenericRelationshipList, RelatedCampaign,
        RelatedPackageRefs, RelatedTTP, RelatedThreatActor)
from stix.data_marking import Marking
import stix.utils
from stix.utils import dates
from datetime import datetime
from dateutil.tz import tzutc
from stix.common.vocabs import (ThreatActorType, Motivation, ThreatActorSophistication,
                                IntendedEffect, PlanningAndOperationalSupport)

class ObservedTTPs(GenericRelationshipList):
    _namespace = 'http://stix.mitre.org/ThreatActor-1'
    _binding = threat_actor_binding
    _binding_class = threat_actor_binding.ObservedTTPsType
    _binding_var = "Observed_TTP"
    _contained_type = RelatedTTP
    _inner_name = "ttps"


class AssociatedActors(GenericRelationshipList):
    _namespace = 'http://stix.mitre.org/ThreatActor-1'
    _binding = threat_actor_binding
    _binding_class = threat_actor_binding.AssociatedActorsType
    _binding_var = "Associated_Actor"
    _contained_type = RelatedThreatActor
    _inner_name = "threat_actors"


class AssociatedCampaigns(GenericRelationshipList):
    _namespace = 'http://stix.mitre.org/ThreatActor-1'
    _binding = threat_actor_binding
    _binding_class = threat_actor_binding.AssociatedCampaignsType
    _binding_var = "Associated_Campaign"
    _contained_type = RelatedCampaign
    _inner_name = "campaigns"


class ThreatActor(stix.Entity):
    _binding = threat_actor_binding
    _binding_class = threat_actor_binding.ThreatActorType
    _namespace = 'http://stix.mitre.org/ThreatActor-1'
    _version = "1.1.1"
    _ALL_VERSIONS = ("1.0", "1.0.1", "1.1", "1.1.1")

    def __init__(self, id_=None, idref=None, timestamp=None, title=None, description=None, short_description=None):
        self.id_ = id_ or stix.utils.create_id("threatactor")
        self.idref = idref
        self.version = None
        self.title = title
        self.description = description
        self.short_description = short_description
        self.identity = None
        self.types = None
        self.motivations = None
        self.sophistications = None
        self.intended_effects = None
        self.planning_and_operational_supports = None
        self.handling = None
        self.confidence = None
        self.information_source = None
        self.observed_ttps = ObservedTTPs()
        self.associated_campaigns = AssociatedCampaigns()
        self.associated_actors = AssociatedActors()
        self.related_packages = RelatedPackageRefs()

        if timestamp:
            self.timestamp = timestamp
        else:
            self.timestamp = datetime.now(tzutc()) if not idref else None

    @property
    def id_(self):
        return self._id
    
    @id_.setter
    def id_(self, value):
        if not value:
            self._id = None
        else:
            self._id = value
            self.idref = None
    
    @property
    def version(self):
        return self._version
    
    @version.setter
    def version(self, value):
        if not value:
            self._version = None
        else:
            if value not in self._ALL_VERSIONS:
                msg = ("Version must be one of %s. Found '%s'" %
                      (self._ALL_VERSIONS, value))
                raise ValueError(msg)
            self._version = value
    
    @property
    def idref(self):
        return self._idref
    
    @idref.setter
    def idref(self, value):
        if not value:
            self._idref = None
        else:
            self._idref = value
            self.id_ = None # unset id_ if idref is present
    
    @property
    def timestamp(self):
        return self._timestamp

    @timestamp.setter
    def timestamp(self, value):
        self._timestamp = dates.parse_value(value)

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value):
        self._title = value

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        if value:
            if isinstance(value, StructuredText):
                self._description = value
            else:
                self._description = StructuredText(value=value)
        else:
            self._description = None

    @property
    def short_description(self):
        return self._short_description

    @short_description.setter
    def short_description(self, value):
        if value:
            if isinstance(value, StructuredText):
                self._short_description = value
            else:
                self._short_description = StructuredText(value=value)
        else:
            self._short_description = None

    @property
    def identity(self):
        return self._identity
    
    @identity.setter
    def identity(self, value):
        if not value:
            self._identity = None
        elif isinstance(value, Identity):
            self._identity = value
        else:
            raise ValueError("identity must be instance of stix.common.Identity")

    @property
    def types(self):
        return self._types
    
    @types.setter
    def types(self, value):
        self._types = []
        if not value:
            return
        elif isinstance(value, list):
            for v in value:
                self.add_type(v)
        else:
            self.add_type(value)
            
    def add_type(self, value):
        if not value:
            return
        elif isinstance(value, Statement):
            self.types.append(value)
        else:
            type_ = ThreatActorType(value)
            self.types.append(Statement(value=type_))

    @property
    def motivations(self):
        return self._motivations
    
    @motivations.setter
    def motivations(self, value):
        self._motivations = []
        if not value:
            return
        elif isinstance(value, list):
            for v in value:
                self.add_motivation(v)
        else:
            self.add_motivation(value)
            
    def add_motivation(self, value):
        if not value:
            return
        elif isinstance(value, Statement):
            self.motivations.append(value)
        else:
            motivation = Motivation(value)
            self.motivations.append(Statement(value=motivation))

    @property
    def sophistications(self):
        return self._sophistications
    
    @sophistications.setter
    def sophistications(self, value):
        self._sophistications = []
        if not value:
            return
        elif isinstance(value, list):
            for v in value:
                self.add_sophistication(v)
        else:
            self.add_sophistication(value)
            
    def add_sophistication(self, value):
        if not value:
            return
        elif isinstance(value, Statement):
            self.sophistications.append(value)
        else:
            sophistication = ThreatActorSophistication(value)
            self.sophistications.append(Statement(value=sophistication))

    @property
    def intended_effects(self):
        return self._intended_effects
    
    @intended_effects.setter
    def intended_effects(self, value):
        self._intended_effects = []
        if not value:
            return
        elif isinstance(value, list):
            for v in value:
                self.add_intended_effect(v)
        else:
            self.add_intended_effect(value)
            
    def add_intended_effect(self, value):
        if not value:
            return
        elif isinstance(value, Statement):
            self.intended_effects.append(value)
        else:
            intended_effect = IntendedEffect(value)
            self.intended_effects.append(Statement(value=intended_effect))

    @property
    def planning_and_operational_supports(self):
        return self._planning_and_operational_supports
    
    @planning_and_operational_supports.setter
    def planning_and_operational_supports(self, value):
        self._planning_and_operational_supports = []
        if not value:
            return
        elif isinstance(value, list):
            for v in value:
                self.add_planning_and_operational_support(v)
        else:
            self.add_planning_and_operational_support(value)
            
    def add_planning_and_operational_support(self, value):
        if not value:
            return
        elif isinstance(value, Statement):
            self.planning_and_operational_supports.append(value)
        else:
            pos = PlanningAndOperationalSupport(value)
            self.planning_and_operational_supports.append(Statement(value=pos))

    def to_obj(self, return_obj=None, ns_info=None):
        super(ThreatActor, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        return_obj.id = self.id_
        return_obj.idref = self.idref
        if self.timestamp:
            return_obj.timestamp = dates.serialize_value(self.timestamp)
        return_obj.version = self.version
        return_obj.Title = self.title
        if self.description:
            return_obj.Description = self.description.to_obj(ns_info=ns_info)
        if self.short_description:
            return_obj.Short_Description = self.short_description.to_obj(ns_info=ns_info)
        if self.identity:
            return_obj.Identity = self.identity.to_obj(ns_info=ns_info)
        if self.types:
            return_obj.Type = [x.to_obj(ns_info=ns_info) for x in self.types]
        if self.motivations:
            return_obj.Motivation = [x.to_obj(ns_info=ns_info) for x in self.motivations]
        if self.sophistications:
            return_obj.Sophistication = [x.to_obj(ns_info=ns_info) for x in self.sophistications]
        if self.intended_effects:
            return_obj.Intended_Effect = [x.to_obj(ns_info=ns_info) for x in self.intended_effects]
        if self.planning_and_operational_supports:
            return_obj.Planning_And_Operational_Support = [x.to_obj(ns_info=ns_info) for x in self.planning_and_operational_supports]
        if self.observed_ttps:
            return_obj.Observed_TTPs = self.observed_ttps.to_obj(ns_info=ns_info)
        if self.associated_campaigns:
            return_obj.Associated_Campaigns = self.associated_campaigns.to_obj(ns_info=ns_info)
        if self.associated_actors:
            return_obj.Associated_Actors = self.associated_actors.to_obj(ns_info=ns_info)
        if self.handling:
            return_obj.Handling = self.handling.to_obj(ns_info=ns_info)
        if self.confidence:
            return_obj.Confidence = self.confidence.to_obj(ns_info=ns_info)
        if self.information_source:
            return_obj.Information_Source = self.information_source.to_obj(ns_info=ns_info)
        if self.related_packages:
            return_obj.Related_Packages = self.related_packages.to_obj(ns_info=ns_info)

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.id_ = obj.id
        return_obj.idref = obj.idref
        return_obj.timestamp = obj.timestamp
        if isinstance(obj, cls._binding_class): # ThreatActorType properties
            return_obj.version = obj.version
            return_obj.title = obj.Title
            return_obj.description = StructuredText.from_obj(obj.Description)
            return_obj.short_description = StructuredText.from_obj(obj.Short_Description)
            return_obj.identity = Identity.from_obj(obj.Identity)
            return_obj.types = [Statement.from_obj(x) for x in obj.Type]
            return_obj.motivations = [Statement.from_obj(x) for x in obj.Motivation]
            return_obj.sophistications = [Statement.from_obj(x) for x in obj.Sophistication]
            return_obj.intended_effects = [Statement.from_obj(x) for x in obj.Intended_Effect]
            return_obj.planning_and_operational_supports = [Statement.from_obj(x) for x in obj.Planning_And_Operational_Support]
            return_obj.observed_ttps = ObservedTTPs.from_obj(obj.Observed_TTPs)
            return_obj.associated_campaigns = AssociatedCampaigns.from_obj(obj.Associated_Campaigns)
            return_obj.associated_actors = AssociatedActors.from_obj(obj.Associated_Actors)
            return_obj.handling = Marking.from_obj(obj.Handling)
            return_obj.confidence = Confidence.from_obj(obj.Confidence)
            return_obj.information_source = InformationSource.from_obj(obj.Information_Source)
            return_obj.related_packages = RelatedPackageRefs.from_obj(obj.Related_Packages)

        return return_obj

    def to_dict(self):
        d = {}
        if self.id_:
            d['id'] = self.id_
        if self.idref:
            d['idref'] = self.idref
        if self.timestamp:
            d['timestamp'] = dates.serialize_value(self.timestamp)
        if self.version:
            d['version'] = self.version
        if self.title:
            d['title'] = self.title
        if self.description:
            d['description'] = self.description.to_dict()
        if self.short_description:
            d['short_description'] = self.short_description.to_dict()
        if self.identity:
            d['identity'] = self.identity.to_dict()
        if self.types:
            d['types'] = [x.to_dict() for x in self.types]
        if self.motivations:
            d['motivations'] = [x.to_dict() for x in self.motivations]
        if self.sophistications:
            d['sophistications'] = [x.to_dict() for x in self.sophistications]
        if self.intended_effects:
            d['intended_effects'] = [x.to_dict() for x in self.intended_effects]
        if self.planning_and_operational_supports:
            d['planning_and_operational_supports'] = [x.to_dict()
                    for x in self.planning_and_operational_supports]
        if self.observed_ttps:
            d['observed_ttps'] = self.observed_ttps.to_dict()
        if self.associated_campaigns:
            d['associated_campaigns'] = self.associated_campaigns.to_dict()
        if self.associated_actors:
            d['associated_actors'] = self.associated_actors.to_dict()
        if self.handling:
            d['handling'] = self.handling.to_dict()
        if self.confidence:
            d['confidence'] = self.confidence.to_dict()
        if self.information_source:
            d['information_source'] = self.information_source.to_dict()
        if self.related_packages:
            d['related_packages'] = self.related_packages.to_dict()

        return d

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.id_ = dict_repr.get('id')
        return_obj.idref = dict_repr.get('idref')
        return_obj.timestamp = dict_repr.get('timestamp')
        return_obj.version = dict_repr.get('version')
        return_obj.title = dict_repr.get('title')
        return_obj.description = StructuredText.from_dict(dict_repr.get('description'))
        return_obj.short_description = StructuredText.from_dict(dict_repr.get('short_description'))
        return_obj.identity = Identity.from_dict(dict_repr.get('identity'))
        return_obj.types = [Statement.from_dict(x) for x in dict_repr.get('types', [])]
        return_obj.motivations = [Statement.from_dict(x) for x in dict_repr.get('motivations', [])]
        return_obj.sophistications = [Statement.from_dict(x) for x in dict_repr.get('sophistications', [])]
        return_obj.intended_effects = [Statement.from_dict(x) for x in dict_repr.get('intended_effects', [])]
        return_obj.planning_and_operational_supports = [Statement.from_dict(x)
                for x in dict_repr.get('planning_and_operational_supports', [])]
        return_obj.observed_ttps = ObservedTTPs.from_dict(dict_repr.get('observed_ttps'))
        return_obj.associated_campaigns = AssociatedCampaigns.from_dict(dict_repr.get('associated_campaigns'))
        return_obj.associated_actors = AssociatedActors.from_dict(dict_repr.get('associated_actors'))
        return_obj.handling = Marking.from_dict(dict_repr.get('handling'))
        return_obj.confidence = Confidence.from_dict(dict_repr.get('confidence'))
        return_obj.information_source = InformationSource.from_dict(dict_repr.get('information_source'))
        return_obj.related_packages = RelatedPackageRefs.from_dict(dict_repr.get('related_packages'))

        return return_obj



#!/usr/bin/env python
# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

'''
Prints certain fields from a STIXPackage object using the python interface
'''
import sys

from stix.core import STIXPackage
from stix.indicator import Indicator
from cybox.core import Object, Observable

def main():
    fn = sys.argv[-1] if len(sys.argv) == 2 else 'sample.xml'
    stix_package = STIXPackage.from_xml(fn)

    indicator_count = 0
    observable_count = 0
    object_count = 0
    for child in stix_package.walk():
        if isinstance(child, Indicator):
            indicator_count += 1
        elif isinstance(child, Observable):
            observable_count += 1
        elif isinstance(child, Object):
            object_count += 1

    print "Indicators:", indicator_count
    print "Observables:", observable_count
    print "Objects:", object_count

if __name__ == '__main__':
    main()




# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest
from lxml import etree
import stix.utils as utils

class CdataTests(unittest.TestCase):
    def test_strip_cdata(self):
        initial = "TESTTESTTEST"
        wrapped = "<![CDATA[%s]]>" % (initial)
        stripped = utils.strip_cdata(wrapped)

        self.assertEqual(stripped, initial)

    def test_wrap_cdata(self):
        initial = "TESTTESTTEST"
        wrapped = "<![CDATA[%s]]>" % (initial)
        cdata = utils.cdata(initial)

        self.assertEqual(wrapped, cdata)

    def test_round_trip(self):
        initial = "TESTTESTTEST"
        round_trip = utils.strip_cdata(utils.cdata(initial))

        self.assertEqual(initial, round_trip)

    def test_multi_cdata_strip(self):
        initial = "TESTTESTTEST"
        multi = "<![CDATA[{0}]]><![CDATA[{0}]]>".format(initial)
        stripped = utils.strip_cdata(multi)
        self.assertEqual(stripped, initial*2)

if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.utils
import stix.bindings.incident as incident_binding
from cybox.common import Contributor

class Contributors(stix.EntityList):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding = incident_binding
    _binding_class = _binding.ContributorsType
    _contained_type = Contributor
    _binding_var = "Contributor"
    _inner_name = "contributors"
    


# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.utils
from stix.common import StructuredText
import stix.bindings.ttp as ttp_binding

class AttackPattern(stix.Entity):
    _binding = ttp_binding
    _binding_class = _binding.AttackPatternType
    _namespace = "http://stix.mitre.org/TTP-1"

    def __init__(self, id_=None, title=None, description=None, short_description=None):
        self.id_ = id_
        self.capec_id = None
        self.title = title
        self.description = description
        self.short_description = short_description

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value):
        self._title = value

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        if value:
            if isinstance(value, StructuredText):
                self._description = value
            else:
                self._description = StructuredText(value=value)
        else:
            self._description = None

    @property
    def short_description(self):
        return self._short_description

    @short_description.setter
    def short_description(self, value):
        if value:
            if isinstance(value, StructuredText):
                self._short_description = value
            else:
                self._short_description = StructuredText(value=value)
        else:
            self._short_description = None

    def to_obj(self, return_obj=None, ns_info=None):
        super(AttackPattern, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        return_obj.id = self.id_
        return_obj.capec_id = self.capec_id
        return_obj.Title = self.title

        if self.description:
            return_obj.Description = self.description.to_obj(ns_info=ns_info)
        if self.short_description:
            return_obj.Short_Description = self.short_description.to_obj(ns_info=ns_info)

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.id_ = obj.id
        return_obj.capec_id = obj.capec_id
        return_obj.title = obj.Title
        return_obj.description = StructuredText.from_obj(obj.Description)
        return_obj.short_description = StructuredText.from_obj(obj.Short_Description)

        return return_obj

    def to_dict(self):
        d = {}
        if self.id_:
            d['id'] = self.id_
        if self.capec_id:
            d['capec_id'] = self.capec_id
        if self.title:
            d['title'] = self.title
        if self.description:
            d['description'] = self.description.to_dict()
        if self.short_description:
            d['short_description'] = self.short_description.to_dict()

        return d

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.id_ = dict_repr.get('id')
        return_obj.capec_id = dict_repr.get('capec_id')
        return_obj.title = dict_repr.get('title')
        return_obj.description = StructuredText.from_dict(dict_repr.get('description'))
        return_obj.short_description = StructuredText.from_dict(dict_repr.get('short_description'))

        return return_obj



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.bindings.exploit_target as exploit_target_binding
from stix.common import DateTimeWithPrecision, StructuredText
from stix.common.related import GenericRelationshipList, RelatedObservable

# TODO document these
class Vulnerability(stix.Entity):
    """Implementation of STIX ``Vulnerability``.

    Args:
        title (optional): A string title.
        description (optional): A string description.
        short_description (optional): A string short description.

    """
    
    _binding = exploit_target_binding
    _binding_class = _binding.VulnerabilityType
    _namespace = "http://stix.mitre.org/ExploitTarget-1"

    def __init__(self, title=None, description=None, short_description=None):
        self.is_known = None
        self.is_publicly_acknowledged = None
        self.title = title
        self.description = description
        self.short_description = short_description
        self.cve_id = None
        self.osvdb_id = None
        self.source = None
        self.cvss_score = None
        self.discovered_datetime = None
        self.published_datetime = None
        self.affected_software = AffectedSoftware()
        # references

    @property
    def title(self):
        """
        String representing the Vulnerability Title
        """
        return self._title

    @title.setter
    def title(self, value):
        self._title = value

    @property
    def description(self):
        """The ``description`` property for this :class:`Vulnerability`.

        Default Value: ``None``

        Note:
            If set to a value that is not an instance of
            :class:`stix.common.structured_text.StructuredText`, an attempt to
            will be made to convert the value into an instance of
            :class:`stix.common.structured_text.StructuredText`.

        Returns:
            An instance of
            :class:`stix.common.structured_text.StructuredText`

        """

        return self._description

    @description.setter
    def description(self, value):
        
        if value:
            if isinstance(value, StructuredText):
                self._description = value
            else:
                self._description = StructuredText(value=value)
        else:
            self._description = None

    @property
    def short_description(self):
        """The ``short_description`` property for this :class:`Vulnerability`.

        Default Value: ``None``

        Note:
            If set to a value that is not an instance of
            :class:`stix.common.structured_text.StructuredText`, an attempt to
            will be made to convert the value into an instance of
            :class:`stix.common.structured_text.StructuredText`.

        Returns:
            An instance of
            :class:`stix.common.structured_text.StructuredText`

        """
        return self._short_description

    @short_description.setter
    def short_description(self, value):
        if value:
            if isinstance(value, StructuredText):
                self._short_description = value
            else:
                self._short_description = StructuredText(value=value)
        else:
            self._short_description = None

    @property
    def discovered_datetime(self):
        """
        Returns: 
            The time this vulnerability was discovered, represented as 
            class:`DateTimeWithPrecision` 
        """
        return self._discovered_datetime

    @discovered_datetime.setter
    def discovered_datetime(self, value):
        """
        Sets the time this vulnerability was discovered, represented as 
        class:`DateTimeWithPrecision` 

        Default Value: ``None``

        Returns: 
            None

        """
        if value:
            if isinstance(value, DateTimeWithPrecision):
                self._discovered_datetime = value
            else:
                self._discovered_datetime = DateTimeWithPrecision(value=value)
        else:
            self._discovered_datetime = None

    def to_obj(self, return_obj=None, ns_info=None):
        super(Vulnerability, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        if self.is_known is not None:
            return_obj.is_known = self.is_known
        if self.is_publicly_acknowledged is not None:
            return_obj.is_publicly_acknowledged = self.is_publicly_acknowledged
        return_obj.Title = self.title
        if self.description:
            return_obj.Description = self.description.to_obj(ns_info=ns_info)
        if self.short_description:
            return_obj.Short_Description = self.short_description.to_obj(ns_info=ns_info)
        return_obj.CVE_ID = self.cve_id
        return_obj.OSVDB_ID = self.osvdb_id
        return_obj.Source = self.source
        if self.cvss_score:
            return_obj.CVSS_Score = self.cvss_score.to_obj(ns_info=ns_info)
        if self.discovered_datetime:
            return_obj.Discovered_DateTime = self.discovered_datetime.to_obj(ns_info=ns_info)
        if self.published_datetime:
            return_obj.Published_DateTime = self.published_datetime.to_obj(ns_info=ns_info)
        if self.affected_software:
            return_obj.Affected_Software = self.affected_software.to_obj(ns_info=ns_info)

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.is_known = obj.is_known
        return_obj.is_publicly_acknowledged = obj.is_publicly_acknowledged
        return_obj.title = obj.Title
        return_obj.description = StructuredText.from_obj(obj.Description)
        return_obj.short_description = StructuredText.from_obj(obj.Short_Description)
        return_obj.cve_id = obj.CVE_ID
        return_obj.osvdb_id = obj.OSVDB_ID
        return_obj.source = obj.Source
        return_obj.cvss_score = CVSSVector.from_obj(obj.CVSS_Score)
        return_obj.discovered_datetime = DateTimeWithPrecision.from_obj(obj.Discovered_DateTime)
        return_obj.published_datetime = DateTimeWithPrecision.from_obj(obj.Published_DateTime)
        return_obj.affected_software = AffectedSoftware.from_obj(obj.Affected_Software)

        return return_obj

    def to_dict(self):
        d = {}

        if self.is_known is not None:
            d['is_known'] = self.is_known
        if self.is_publicly_acknowledged is not None:
            d['is_publicly_acknowledged'] = self.is_publicly_acknowledged
        if self.title:
            d['title'] = self.title
        if self.description:
            d['description'] = self.description.to_dict()
        if self.short_description:
            d['short_description'] = self.short_description.to_dict()
        if self.cve_id:
            d['cve_id'] = self.cve_id
        if self.osvdb_id:
            d['osvdb_id'] = self.osvdb_id
        if self.source:
            d['source'] = self.source
        if self.cvss_score:
            d['cvss_score'] = self.cvss_score.to_dict()
        if self.discovered_datetime:
            d['discovered_datetime'] = self.discovered_datetime.to_dict()
        if self.published_datetime:
            d['published_datetime'] = self.published_datetime.to_dict()
        if self.affected_software:
            d['affected_software'] = self.affected_software.to_dict()

        return d

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.is_known = dict_repr.get('is_known')
        return_obj.is_publicly_acknowledged = dict_repr.get('is_publicly_acknowledged')
        return_obj.title = dict_repr.get('title')
        return_obj.description = StructuredText.from_dict(dict_repr.get('description'))
        return_obj.short_description = StructuredText.from_dict(dict_repr.get('short_description'))
        return_obj.cve_id = dict_repr.get('cve_id')
        return_obj.osvdb_id = dict_repr.get('osvdb_id')
        return_obj.source = dict_repr.get('source')
        return_obj.cvss_score = CVSSVector.from_dict(dict_repr.get('cvss_score'))
        return_obj.discovered_datetime = DateTimeWithPrecision.from_dict(dict_repr.get('discovered_datetime'))
        return_obj.published_datetime = DateTimeWithPrecision.from_dict(dict_repr.get('published_datetime'))
        return_obj.affected_software = AffectedSoftware.from_dict(dict_repr.get('affected_software'))

        return return_obj


class CVSSVector(stix.Entity):
    """
    Common Vulnerabilit Scoring System object, representing its component measures
    
    """
    _binding = exploit_target_binding
    _binding_class = exploit_target_binding.CVSSVectorType
    _namespace = "http://stix.mitre.org/ExploitTarget-1"

    def __init__(self):
        self.overall_score = None
        self.base_score = None
        self.base_vector = None
        self.temporal_score = None
        self.temporal_vector = None
        self.environmental_score = None
        self.environmental_vector = None

    def to_obj(self, return_obj=None, ns_info=None):
        super(CVSSVector, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        return_obj.Overall_Score = self.overall_score
        return_obj.Base_Score = self.base_score
        return_obj.Base_Vector = self.base_vector
        return_obj.Temporal_Score = self.temporal_score
        return_obj.Temporal_Vector = self.temporal_vector
        return_obj.Environmental_Score = self.environmental_score
        return_obj.Environmental_Vector = self.environmental_vector

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.overall_score = obj.Overall_Score
        return_obj.base_score = obj.Base_Score
        return_obj.base_vector = obj.Base_Vector
        return_obj.temporal_score = obj.Temporal_Score
        return_obj.temporal_vector = obj.Temporal_Vector
        return_obj.environmental_score = obj.Environmental_Score
        return_obj.environmental_vector = obj.Environmental_Vector

        return return_obj

    def to_dict(self):
        d = {}

        if self.overall_score:
            d['overall_score'] = self.overall_score
        if self.base_score:
            d['base_score'] = self.base_score
        if self.base_vector:
            d['base_vector'] = self.base_vector
        if self.temporal_score:
            d['temporal_score'] = self.temporal_score
        if self.temporal_vector:
            d['temporal_vector'] = self.temporal_vector
        if self.environmental_score:
            d['environmental_score'] = self.environmental_score
        if self.environmental_vector:
            d['environmental_vector'] = self.environmental_vector

        return d

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.overall_score = dict_repr.get('overall_score')
        return_obj.base_score = dict_repr.get('base_score')
        return_obj.base_vector = dict_repr.get('base_vector')
        return_obj.temporal_score = dict_repr.get('temporal_score')
        return_obj.temporal_vector = dict_repr.get('temporal_vector')
        return_obj.environmental_score = dict_repr.get('environmental_score')
        return_obj.environmental_vector = dict_repr.get('environmental_vector')

        return return_obj


class AffectedSoftware(GenericRelationshipList):
    _binding = exploit_target_binding
    _binding_class = exploit_target_binding.AffectedSoftwareType
    _namespace = "http://stix.mitre.org/ExploitTarget-1"
    _binding_var = "Affected_Software"
    _contained_type = RelatedObservable
    _inner_name = "affected_software"



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.utils
import stix.indicator.test_mechanism
from stix.common import EncodedCDATA
from stix.indicator.test_mechanism import _BaseTestMechanism
import stix.bindings.extensions.test_mechanism.snort as snort_tm_binding

class SnortTestMechanism(_BaseTestMechanism):
    _namespace = "http://stix.mitre.org/extensions/TestMechanism#Snort-1"
    _binding = snort_tm_binding
    _binding_class = _binding.SnortTestMechanismType
    _XSI_TYPE = "snortTM:SnortTestMechanismType"
    
    def __init__(self, id_=None, idref=None):
        super(SnortTestMechanism, self).__init__(id_=id_, idref=idref)
        self.product_name = None
        self.version = None
        self.rules = None
        self.event_filters = None
        self.rate_filters = None
        self.event_suppressions = None
    
    @property
    def rules(self):
        return self._rules
    
    @rules.setter
    def rules(self, value):
        self._rules = []
        if not value:
            return
        elif isinstance(value, list):
            for v in value:
                self.add_rule(v)
        else:
            self.add_rule(v)
    
    def add_rule(self, rule):
        if not rule:
            return
        elif isinstance(rule, EncodedCDATA):
            self.rules.append(rule)
        else:
            self.rules.append(EncodedCDATA(value=rule))
    
    @property
    def event_filters(self):
        return self._event_filters
    
    @event_filters.setter
    def event_filters(self, value):
        self._event_filters = []
        if not value:
            return
        elif isinstance(value, list):
            for v in value:
                self.add_event_filter(v)
        else:
            self.add_event_filter(v)
    
    def add_event_filter(self, item):
        if not item:
            return
        elif isinstance(item, EncodedCDATA):
            self.event_filters.append(item)
        else:
            self.rules.append(EncodedCDATA(value=item))  
    
    @property
    def rate_filters(self):
        return self._rate_filters
    
    @rate_filters.setter
    def rate_filters(self, value):
        self._rate_filters = []
        if not value:
            return
        elif isinstance(value, list):
            for v in value:
                self.add_rate_filter(v)
        else:
            self.add_rate_filter(v)
    
    def add_rate_filter(self, item):
        if not item:
            return
        elif isinstance(item, EncodedCDATA):
            self.rate_filters.append(item)
        else:
            self.rules.append(EncodedCDATA(value=item))  
    
    @property
    def event_suppressions(self):
        return self._event_suppressions
    
    @event_suppressions.setter
    def event_suppressions(self, value):
        self._event_suppressions = []
        if not value:
            return
        elif isinstance(value, list):
            for v in value:
                self.add_event_suppression(v)
        else:
            self.add_event_suppression(v)
    
    def add_event_suppression(self, item):
        if not item:
            return
        elif isinstance(item, EncodedCDATA):
            self.event_suppressions.append(item)
        else:
            self.rules.append(EncodedCDATA(value=item))  
    
    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()
        
        super(SnortTestMechanism, cls).from_obj(obj, return_obj)
        return_obj.product_name = obj.Product_Name
        return_obj.version = obj.Version
        
        if obj.Rule:
            return_obj.rules = [EncodedCDATA.from_obj(x) for x in obj.Rule]
        if obj.Event_Filter:
            return_obj.event_filters = [EncodedCDATA.from_obj(x) for x in obj.Event_Filter]
        if obj.Rate_Filter:
            return_obj.rate_filters = [EncodedCDATA.from_obj(x) for x in obj.Rate_Filter]
        if obj.Event_Suppression:
            return_obj.event_suppressions = [EncodedCDATA.from_obj(x) for x in obj.Event_Suppression]
        
        return return_obj
    
    def to_obj(self, return_obj=None, ns_info=None):
        if not return_obj:
            return_obj = self._binding_class()
            
        super(SnortTestMechanism, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        return_obj.Product_Name = self.product_name
        return_obj.Version = self.version
        
        if self.rules:
            return_obj.Rule = [x.to_obj(ns_info=ns_info) for x in self.rules]
        if self.event_filters:
            return_obj.Event_Filter = [x.to_obj(ns_info=ns_info) for x in self.event_filters]
        if self.rate_filters:
            return_obj.Rate_Filter = [x.to_obj(ns_info=ns_info) for x in self.rate_filters]
        if self.event_suppressions:
            return_obj.Event_Suppression = [x.to_obj(ns_info=ns_info) for x in self.event_suppressions]
        
        return return_obj
    
    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        if not return_obj:
            return_obj = cls()
            
        super(SnortTestMechanism, cls).from_dict(d, return_obj)
        
        return_obj.product_name = d.get('product_name')
        return_obj.version = d.get('version')
        return_obj.rules = [EncodedCDATA.from_dict(x) for x in d.get('rules', [])]
        return_obj.event_filters = [EncodedCDATA.from_dict(x) for x in d.get('event_filters', [])]
        return_obj.rate_filters = [EncodedCDATA.from_dict(x) for x in d.get('rate_filters', [])]
        return_obj.event_suppressions = [EncodedCDATA.from_dict(x) for x in d.get('event_suppressions', [])]
        
        return return_obj


    def to_dict(self):
        d = super(SnortTestMechanism, self).to_dict()

        if self.product_name:
            d['product_name'] = self.product_name
        if self.version:
            d['version'] = self.version
        if self.rules:
            d['rules'] = [x.to_dict() for x in self.rules]
        if self.event_filters:
            d['event_filters'] = [x.to_dict() for x in self.event_filters]
        if self.rate_filters:
            d['rate_filters'] = [x.to_dict() for x in self.rate_filters]
        if self.event_suppressions:
            d['event_suppressions'] = [x.to_dict for x in self.event_suppressions]

        return d

stix.indicator.test_mechanism.add_extension(SnortTestMechanism)



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.bindings.incident as incident_binding
from stix.common.datetimewithprecision import DATETIME_PRECISION_VALUES
from stix.utils.dates import parse_value, serialize_value
from .coa import COATaken

class JournalEntry(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding = incident_binding
    _binding_class = incident_binding.JournalEntryType
    
    def __init__(self, value=None):
        self.value = value
        self.author = None
        self.time = None # consider 
        self.time_precision = 'second'
        
    @property
    def time(self):
        return self._time
    
    @time.setter
    def time(self, value):
        self._time = parse_value(value)
        
    @property
    def time_precision(self):
        return self._time_precision
    
    @time_precision.setter
    def time_precision(self, value):
        if value and (value not in DATETIME_PRECISION_VALUES):
            raise ValueError("value must be one of [%s]" % ", ".join(x for x in DATETIME_PRECISION_VALUES))
        self._time_precision = value
    
    def to_obj(self, return_obj=None, ns_info=None):
        super(JournalEntry, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()
        
        return_obj.valueOf_ = self.value
        return_obj.author = self.author
        return_obj.time = serialize_value(self.time)
        return_obj.time_precision = self.time_precision
        
        return return_obj
    
    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        
        if not return_obj:
            return_obj = cls()
            
        return_obj.value = obj.valueOf_
        return_obj.author = obj.author
        return_obj.time = obj.time
        return_obj.time_precision = obj.time_precision
        
        return return_obj
        
    def to_dict(self):
        d = {}
        
        if self.value:
            d['value'] = self.value
        if self.author:
            d['author'] = self.author
        if self.time:
            d['time'] = serialize_value(self.time)
        if self.time_precision:
            d['time_precision'] = self.time_precision
            
        return d

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        
        if not return_obj:
            return_obj = cls()
            
        return_obj.value = d.get('value')
        return_obj.author = d.get('author')
        return_obj.time = d.get('time')
        return_obj.time_precision = d.get('time_precision')
        
        return return_obj

class HistoryItem(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding = incident_binding
    _binding_class = incident_binding.HistoryItemType
    
    def __init__(self):
        self.action_entry = None
        self.journal_entry = None
        
    @property
    def action_entry(self):
        return self._action_entry
    
    @action_entry.setter
    def action_entry(self, value):
        if not value:
            self._action_entry = None
        elif isinstance(value, COATaken):
            self._action_entry = value
        else:
            raise ValueError('value must be instance of stix.incident.COATaken')
    
    @property
    def journal_entry(self):
        return self._journal_entry
    
    @journal_entry.setter
    def journal_entry(self, value):
        if not value:
            self._journal_entry = None
        elif isinstance(value, JournalEntry):
            self._journal_entry = value
        else:
            self._journal_entry = JournalEntry(value)
            
    def to_obj(self, return_obj=None, ns_info=None):
        super(HistoryItem, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()
            
        if self.action_entry:
            return_obj.Action_Entry = self.action_entry.to_obj(ns_info=ns_info)
        if self.journal_entry:
            return_obj.Journal_Entry = self.journal_entry.to_obj(ns_info=ns_info)
        
        return return_obj
    
    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        
        if not return_obj:
            return_obj = cls()
            
        return_obj.action_entry = COATaken.from_obj(obj.Action_Entry)
        return_obj.journal_entry = JournalEntry.from_obj(obj.Journal_Entry)
        
        return return_obj
            
    def to_dict(self):
        d = {}
        
        if self.journal_entry:
            d['journal_entry'] = self.journal_entry.to_dict()
        if self.action_entry:
            d['action_entry'] = self.action_entry.to_dict()
        
        return d
    
    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        
        if not return_obj:
            return_obj = cls()
            
        return_obj.action_entry = COATaken.from_dict(d.get('action_entry'))
        return_obj.journal_entry = JournalEntry.from_dict(d.get('journal_entry'))
        
        return return_obj
    
class History(stix.EntityList):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding = incident_binding
    _binding_class = incident_binding.HistoryType
    _binding_var = "History_Item"
    _contained_type = HistoryItem
    _inner_name = "history_items"



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from datetime import datetime
from stix.utils import dates
from dateutil.tz import tzutc
from cybox.core import Observables
import stix
import stix.utils
import stix.bindings.course_of_action as coa_binding
from stix.common import StructuredText, VocabString, InformationSource, Statement
from stix.common.related import (GenericRelationshipList, RelatedCOA,
        RelatedPackageRefs)
from stix.data_marking import Marking
from .objective import Objective

from stix.common.vocabs import COAStage as Stage
from stix.common.vocabs import CourseOfActionType as COAType

class RelatedCOAs(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/CourseOfAction-1"
    _binding = coa_binding
    _binding_class = coa_binding.RelatedCOAsType
    _binding_var = "Related_COA"
    _contained_type = RelatedCOA
    _inner_name = "coas"

class CourseOfAction(stix.Entity):
    _binding = coa_binding
    _binding_class = coa_binding.CourseOfActionType
    _namespace = "http://stix.mitre.org/CourseOfAction-1"
    _version = "1.1.1"
    _ALL_VERSIONS = ("1.0", "1.0.1", "1.1", "1.1.1")

    def __init__(self, id_=None, idref=None, timestamp=None, title=None, description=None, short_description=None):
        self.id_ = id_ or stix.utils.create_id("coa")
        self.idref = idref
        self.version = None
        self.title = title
        self.stage = None
        self.type_ = None
        self.description = description
        self.short_description = short_description
        self.objective = None
        self.parameter_observables = None
        # self.structured_coa = None
        self.impact = None
        self.cost = None
        self.efficacy = None
        self.information_source = None
        self.handling = None
        self.related_coas = RelatedCOAs()
        self.related_packages = RelatedPackageRefs()

        if timestamp:
            self.timestamp = timestamp
        else:
            self.timestamp = datetime.now(tzutc()) if not idref else None

    @property
    def id_(self):
        return self._id
    
    @id_.setter
    def id_(self, value):
        if not value:
            self._id = None
        else:
            self._id = value
            self.idref = None
    
    @property
    def version(self):
        return self._version
    
    @version.setter
    def version(self, value):
        if not value:
            self._version = None
        else:
            if value not in self._ALL_VERSIONS:
                msg = ("Version must be one of %s. Found '%s'" %
                      (self._ALL_VERSIONS, value))
                raise ValueError(msg)
            self._version = value
    
    @property
    def idref(self):
        return self._idref
    
    @idref.setter
    def idref(self, value):
        if not value:
            self._idref = None
        else:
            self._idref = value
            self.id_ = None # unset id_ if idref is present

    @property
    def timestamp(self):
        return self._timestamp

    @timestamp.setter
    def timestamp(self, value):
        self._timestamp = dates.parse_value(value)

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value):
        self._title = value

    @property
    def stage(self):
        return self._stage

    @stage.setter
    def stage(self, value):
        if not value:
            self._stage = None
        elif isinstance(value, VocabString):
            self._stage = value
        else:
            self._stage = Stage(value=value)

    @property
    def type_(self):
        return self._type_

    @type_.setter
    def type_(self, value):
        if not value:
            self._type_ = None
        elif isinstance(value, VocabString):
            self._type_ = value
        else:
            self._type_ = COAType(value=value)

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        if value:
            if isinstance(value, StructuredText):
                self._description = value
            else:
                self._description = StructuredText(value=value)
        else:
            self._description = None

    @property
    def short_description(self):
        return self._short_description

    @short_description.setter
    def short_description(self, value):
        if value:
            if isinstance(value, StructuredText):
                self._short_description = value
            else:
                self._short_description = StructuredText(value=value)
        else:
            self._short_description = None

    @property
    def objective(self):
        return self._objective

    @objective.setter
    def objective(self, value):
        if not value:
            self._objective = None
        elif isinstance(value, Objective):
            self._objective = value
        else:
            raise ValueError('Cannot set objective to type %s' % type(value))

    @property
    def impact(self):
        return self._impact

    @impact.setter
    def impact(self, impact):
        if not impact:
            self._impact = None
        elif isinstance(impact, Statement):
            self._impact = impact
        else:
            self._impact = Statement(value=impact)

    @property
    def cost(self):
        return self._cost

    @cost.setter
    def cost(self, cost):
        if not cost:
            self._cost = None
        elif isinstance(cost, Statement):
            self._cost = cost
        else:
            self._cost = Statement(value=cost)

    @property
    def efficacy(self):
        return self._efficacy

    @efficacy.setter
    def efficacy(self, efficacy):
        if not efficacy:
            self._efficacy = None
        elif isinstance(efficacy, Statement):
            self._efficacy = efficacy
        else:
            self._efficacy = Statement(value=efficacy)

    @property
    def information_source(self):
        return self._information_source

    @information_source.setter
    def information_source(self, value):
        if not value:
            self._information_source = None
        elif isinstance(value, InformationSource):
            self._information_source = value
        else:
            raise ValueError('value must be instance of InformationSource')

    @property
    def handling(self):
        return self._handling

    @handling.setter
    def handling(self, value):
        if value and not isinstance(value, Marking):
            raise ValueError('value must be instance of Marking')

        self._handling = value

    def to_obj(self, return_obj=None, ns_info=None):
        super(CourseOfAction, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        return_obj.id = self.id_
        return_obj.idref = self.idref
        if self.timestamp:
            return_obj.timestamp = self.timestamp.isoformat()
        return_obj.version = self.version
        return_obj.Title = self.title
        if self.stage:
            return_obj.Stage = self.stage.to_obj(ns_info=ns_info)
        if self.type_:
            return_obj.Type = self.type_.to_obj(ns_info=ns_info)
        if self.description:
            return_obj.Description = self.description.to_obj(ns_info=ns_info)
        if self.short_description:
            return_obj.Short_Description = self.short_description.to_obj(ns_info=ns_info)
        if self.objective:
            return_obj.Objective = self.objective.to_obj(ns_info=ns_info)
        if self.parameter_observables:
            return_obj.Parameter_Observables = self.parameter_observables.to_obj(ns_info=ns_info)
        if self.impact:
            return_obj.Impact = self.impact.to_obj(ns_info=ns_info)
        if self.cost:
            return_obj.Cost = self.cost.to_obj(ns_info=ns_info)
        if self.efficacy:
            return_obj.Efficacy = self.efficacy.to_obj(ns_info=ns_info)
        if self.information_source:
            return_obj.Information_Source = self.information_source.to_obj(ns_info=ns_info)
        if self.handling:
            return_obj.Handling = self.handling.to_obj(ns_info=ns_info)
        if self.related_coas:
            return_obj.Related_COAs = self.related_coas.to_obj(ns_info=ns_info)
        if self.related_packages:
            return_obj.Related_Packages = self.related_packages.to_obj(ns_info=ns_info)

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.id_ = obj.id
        return_obj.idref = obj.idref
        return_obj.timestamp = obj.timestamp

        if isinstance(obj, cls._binding_class): # CourseOfActionType properties
            return_obj.version = obj.version
            return_obj.title = obj.Title
            return_obj.stage = VocabString.from_obj(obj.Stage)
            return_obj.type_ = VocabString.from_obj(obj.Type)
            return_obj.description = StructuredText.from_obj(obj.Description)
            return_obj.short_description = StructuredText.from_obj(obj.Short_Description)
            return_obj.objective = Objective.from_obj(obj.Objective)
            return_obj.parameter_observables = \
                    Observables.from_obj(obj.Parameter_Observables)
            return_obj.impact = Statement.from_obj(obj.Impact)
            return_obj.cost = Statement.from_obj(obj.Cost)
            return_obj.efficacy = Statement.from_obj(obj.Efficacy)
            return_obj.information_source = InformationSource.from_obj(obj.Information_Source)
            return_obj.handling = Marking.from_obj(obj.Handling)
            return_obj.related_coas = \
                    RelatedCOAs.from_obj(obj.Related_COAs)
            return_obj.related_packages = \
                    RelatedPackageRefs.from_obj(obj.Related_Packages)

        return return_obj

    def to_dict(self):
        d = {}
        if self.id_:
            d['id'] = self.id_
        if self.idref:
            d['idref'] = self.idref
        if self.timestamp:
            d['timestamp'] = self.timestamp.isoformat()
        if self.version:
            d['version'] = self.version
        if self.title:
            d['title'] = self.title
        if self.stage:
            d['stage'] = self.stage.to_dict()
        if self.type_:
            d['type'] = self.type_.to_dict()
        if self.description:
            d['description'] = self.description.to_dict()
        if self.short_description:
            d['short_description'] = self.short_description.to_dict()
        if self.objective:
            d['objective'] = self.objective.to_dict()
        if self.parameter_observables:
            d['parameter_observables'] = self.parameter_observables.to_dict()
        if self.impact:
            d['impact'] = self.impact.to_dict()
        if self.cost:
            d['cost'] = self.cost.to_dict()
        if self.efficacy:
            d['efficacy'] = self.efficacy.to_dict()
        if self.type_:
            d['type'] = self.type_.to_dict()
        if self.information_source:
            d['information_source'] = self.information_source.to_dict()
        if self.handling:
            d['handling'] = self.handling.to_dict()
        if self.related_coas:
            d['related_coas'] = self.related_coas.to_dict()
        if self.related_packages:
            d['related_packages'] = self.related_packages.to_dict()

        return d

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.id_ = dict_repr.get('id')
        return_obj.idref = dict_repr.get('idref')
        return_obj.timestamp = dict_repr.get('timestamp')
        return_obj.version = dict_repr.get('version')
        return_obj.title = dict_repr.get('title')
        return_obj.stage = VocabString.from_dict(dict_repr.get('stage'))
        return_obj.type_ = VocabString.from_dict(dict_repr.get('type'))
        return_obj.description = StructuredText.from_dict(dict_repr.get('description'))
        return_obj.short_description = StructuredText.from_dict(dict_repr.get('short_description'))
        return_obj.objective = Objective.from_dict(dict_repr.get('objective'))
        return_obj.parameter_observables = \
                Observables.from_dict(dict_repr.get('parameter_observables'))
        return_obj.impact = Statement.from_dict(dict_repr.get('impact'))
        return_obj.cost = Statement.from_dict(dict_repr.get('cost'))
        return_obj.efficacy = Statement.from_dict(dict_repr.get('efficacy'))
        return_obj.information_source = InformationSource.from_dict(dict_repr.get('information_source'))
        return_obj.handling = Marking.from_dict(dict_repr.get('handling'))
        return_obj.related_coas = \
                RelatedCOAs.from_dict(dict_repr.get('related_coas'))
        return_obj.related_packages = \
                RelatedPackageRefs.from_dict(dict_repr.get('related_packages'))

        return return_obj

# alias for CourseOfAction
COA = CourseOfAction


import stix.bindings.extensions.marking.terms_of_use_marking as tou_marking_binding
import stix.data_marking
from stix.data_marking import MarkingStructure


class TermsOfUseMarkingStructure(MarkingStructure):
    _binding = tou_marking_binding
    _binding_class = tou_marking_binding.TermsOfUseMarkingStructureType
    _namespace = 'http://data-marking.mitre.org/extensions/MarkingStructure#Terms_Of_Use-1'
    _XSI_TYPE = "TOUMarking:TermsOfUseMarkingStructureType"

    def __init__(self, terms_of_use=None):
        super(TermsOfUseMarkingStructure, self).__init__()
        self.terms_of_use = terms_of_use

    def to_obj(self, return_obj=None, ns_info=None):
        super(TermsOfUseMarkingStructure, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        obj = self._binding_class()

        MarkingStructure.to_obj(self, return_obj=obj, ns_info=ns_info)

        obj.Terms_Of_Use = self.terms_of_use

        return obj

    def to_dict(self):
        d = MarkingStructure.to_dict(self)
        if self.terms_of_use:
            d['terms_of_use'] = self.terms_of_use

        return d

    @staticmethod
    def from_obj(obj):
        if not obj:
            return None

        m = TermsOfUseMarkingStructure()
        MarkingStructure.from_obj(obj, m)
        m.terms_of_use = obj.Terms_Of_Use

        return m

    @staticmethod
    def from_dict(marking_dict):
        if not marking_dict:
            return None

        m = TermsOfUseMarkingStructure()
        MarkingStructure.from_dict(marking_dict, m)
        m.terms_of_use = marking_dict.get('terms_of_use')

        return m


stix.data_marking.add_extension(TermsOfUseMarkingStructure)




# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
from stix.common.vocabs import SecurityCompromise
import stix.bindings.incident as incident_binding
from stix.common import VocabString

class IndirectImpactSummary(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding  = incident_binding
    _binding_class = incident_binding.IndirectImpactSummaryType

    def __init__(self):
        super(IndirectImpactSummary, self).__init__()
        self.loss_of_competitive_advantage = None
        self.brand_and_market_damage = None
        self.increased_operating_costs = None
        self.legal_and_regulatory_costs = None

    @property
    def loss_of_competitive_advantage(self):
        return self._loss_of_competitive_advantage

    @loss_of_competitive_advantage.setter
    def loss_of_competitive_advantage(self, value):
        if not value:
            self._loss_of_competitive_advantage = None
        elif isinstance(value, VocabString):
            self._loss_of_competitive_advantage = value
        else:
            self._loss_of_competitive_advantage = SecurityCompromise(value=value)

    @property
    def brand_and_market_damage(self):
        return self._brand_and_market_damage

    @brand_and_market_damage.setter
    def brand_and_market_damage(self, value):
        if not value:
            self._brand_and_market_damage = None
        elif isinstance(value, VocabString):
            self._brand_and_market_damage = value
        else:
            self._brand_and_market_damage = SecurityCompromise(value=value)

    @property
    def increased_operating_costs(self):
        return self._increased_operating_costs

    @increased_operating_costs.setter
    def increased_operating_costs(self, value):
        if not value:
            self._increased_operating_costs = None
        elif isinstance(value, VocabString):
            self._increased_operating_costs = value
        else:
            self._increased_operating_costs = SecurityCompromise(value=value)

    @property
    def legal_and_regulatory_costs(self):
        return self._legal_and_regulatory_costs

    @legal_and_regulatory_costs.setter
    def legal_and_regulatory_costs(self, value):
        if not value:
            self._legal_and_regulatory_costs = None
        elif isinstance(value, VocabString):
            self._legal_and_regulatory_costs = value
        else:
            self._legal_and_regulatory_costs = SecurityCompromise(value=value)
            
    def to_obj(self, return_obj=None, ns_info=None):
        super(IndirectImpactSummary, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        obj = self._binding_class()
        if self.loss_of_competitive_advantage:
            obj.Loss_Of_Competitive_Advantage = self.loss_of_competitive_advantage.to_obj(ns_info=ns_info)
        if self.brand_and_market_damage:
            obj.Brand_And_Market_Damage = self.brand_and_market_damage.to_obj(ns_info=ns_info)
        if self.increased_operating_costs:
            obj.Increased_Operating_Costs = self.increased_operating_costs.to_obj(ns_info=ns_info)
        if self.legal_and_regulatory_costs:
            obj.Legal_And_Regulatory_Costs = self.legal_and_regulatory_costs.to_obj(ns_info=ns_info)
        return obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.loss_of_competitive_advantage = VocabString.from_obj(obj.Loss_Of_Competitive_Advantage)
        return_obj.brand_and_market_damage = VocabString.from_obj(obj.Brand_And_Market_Damage)
        return_obj.increased_operating_costs = VocabString.from_obj(obj.Increased_Operating_Costs)
        return_obj.legal_and_regulatory_costs = VocabString.from_obj(obj.Legal_And_Regulatory_Costs)
        return return_obj

    def to_dict(self):    
        d  = {}
        if self.loss_of_competitive_advantage:
            d['loss_of_competitive_advantage'] = self.loss_of_competitive_advantage.to_dict()
        if self.brand_and_market_damage:
            d['brand_and_market_damage'] = self.brand_and_market_damage.to_dict()
        if self.increased_operating_costs:
            d['increased_operating_costs'] = self.increased_operating_costs.to_dict()
        if self.legal_and_regulatory_costs:
            d['legal_and_regulatory_costs'] = self.legal_and_regulatory_costs.to_dict()
        return d

    @classmethod
    def from_dict(cls, dict_, return_obj=None):
        if not dict_:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.loss_of_competitive_advantage = VocabString.from_dict(dict_.get('loss_of_competitive_advantage'))
        return_obj.brand_and_market_damage = VocabString.from_dict(dict_.get('brand_and_market_damage'))
        return_obj.increased_operating_costs = VocabString.from_dict(dict_.get('increased_operating_costs'))
        return_obj.legal_and_regulatory_costs = VocabString.from_dict(dict_.get('legal_and_regulatory_costs'))

        return return_obj



import unittest

from stix.test import EntityTestCase

from stix.core import STIXPackage


class STIXPackageTests(EntityTestCase, unittest.TestCase):
    klass = STIXPackage
    _full_dict = {
        'stix_header': {
            'title': "A Title",
            'description': "A really, really long description",
        },
        'ttps': {
            'ttps': [
            {
                'title': "Foo",
                'version': "1.1.1"
            },
            ]
        },
        'exploit_targets': [
        {
            'id': 'example:ExploitTarget-1',
            'version': '1.1.1',
            'title': "ExploitTarget1",
            'description': "This is a long description about an ExploitTarget",
            'short_description': "an ExploitTarget",
            'information_source': {
                'description': "Mr. Evil's enemy",
                'identity': {
                    'name': "Ms. Good",
                },
            },
            'handling': [
            {
                'marking_structures': [{
                    'marking_model_name': 'TLP',
                    'color': "RED",
                    'xsi:type': "tlpMarking:TLPMarkingStructureType",
                }]
            }
            ]
        }
        ],
        'campaigns': [
            {
                    'id': "example:Campaign-341",
                    'timestamp': "2014-01-31T06:14:46",
                    'version': '1.1.1',
                    'title': 'Purple Elephant',
                    'description': 'A pretty novice set of actors.',
                    'short_description': 'novices',
                    'names': {
                        'names': ["Dancing Hippos", "Crazy Squirrels"],
                    },
                    'intended_effects': [
                        {
                            'timestamp': "2014-03-11T06:24:26",
                            'value': "Doing bad stuff",
                        },
                        {
                            'timestamp': "2014-03-21T06:24:26",
                            'value': "Doing really bad stuff",
                        }
                    ],
                    'status': "Ongoing",
                    'related_ttps': {
                        'scope': "exclusive",
                        'ttps': [
                            {
                                'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                                'ttp': {'title': "Stealth", 'version': "1.1.1"},
                            }
                        ]
                    },
                    'related_incidents': {
                        'scope': "inclusive",
                        'incidents': [
                            {
                                'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                                'incident': {'idref': "example:Incident-2",
                                             'version': "1.1.1"},
                            }
                        ]
                    },
                    'related_indicators': {
                        'scope': "inclusive",
                        'indicators': [
                            {
                                'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                                'indicator': {'idref': "example:Indicator-77",
                                              'version': '2.1.1'},
                            }
                        ]
                    },
                    'attribution': [{
                        'scope': "inclusive",
                        'threat_actors': [
                            {
                                'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                                'threat_actor': {'title': "Campaign Actor #1",
                                                 'version': "1.1.1"},
                            },
                            {
                                'threat_actor': {'idref': "example:ThreatActor-111",
                                                 'version': "1.1.1"},
                            },
                        ],
                    }],
                    'associated_campaigns': {
                        'scope': "inclusive",
                        'campaigns': [
                            {
                                'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                                'information_source': {'description': "Threat Feed"},
                                'campaign': {'title': "Baby Elephant", 'version': "1.1.1"},
                            }
                        ],
                    },
                    'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                    'activity': [
                            {
                                'date_time': "2012-01-01T08:45:31",
                                'description': "The first bad thing"
                            },
                            {
                                'date_time': "2012-01-02T08:45:31",
                                'description': "Another bad thing"
                            },
                    ],
                    'information_source': {
                        'description': "A former member of the campaign.",
                        'identity': {
                            'name': "Mr. D. Fector",
                        },
                    },
                    'handling': [
                        {
                            'marking_structures': [{
                                'marking_model_name': 'TLP',
                                'color': "RED",
                                'xsi:type': "tlpMarking:TLPMarkingStructureType",
                            }]
                        }
                    ],
                    'related_packages': {
                        'packages': [
                            {'idref': "example:Package-AB", 'relationship': "Parent"},
                            {'idref': "example:Package-CD", 'relationship': "Child"}
                        ]
                    }
                }
        ],
        'courses_of_action': [
        {
            'id': 'example:coa-1',
            'version': '1.1.1',
            'timestamp': "2014-03-20T04:35:12",
            'title': "COA1",
            'description': "This is a long description about a course of action",
            'short_description': "a COA",
            'stage': {"value" : "Remedy", "xsi:type" : "stixVocabs:COAStageVocab-1.0"},
            'type': {'value': 'Redirection','xsi:type': 'stixVocabs:CourseOfActionTypeVocab-1.0'},
            'cost': {
                'value': "100"
            },
            'efficacy': {
                'value': "Half"
            },
            'impact': {
                'value': "Large"
            },
            'information_source': {
                'description': "Mr. Evil's enemy",
                'identity': {
                    'name': "Ms. Good",
                },
            },
            'handling': [
            {
                'marking_structures': [{
                    'marking_model_name': 'TLP',
                    'color': "GREEN",
                    'xsi:type': "tlpMarking:TLPMarkingStructureType",
                }]
            }
            ]
        }
        ],
        'version': "1.1.1"
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from stix.campaign import Campaign
from stix.test import EntityTestCase


class CampaignTest(EntityTestCase, unittest.TestCase):
    klass = Campaign
    _full_dict = {
        'id': "example:Campaign-341",
        'timestamp': "2014-01-31T06:14:46",
        'version': '1.1.1',
        'title': 'Purple Elephant',
        'description': 'A pretty novice set of actors.',
        'short_description': 'novices',
        'names': {
            'names': ["Dancing Hippos", "Crazy Squirrels"],
        },
        'intended_effects': [
            {
                'timestamp': "2014-03-11T06:24:26",
                'value': "Doing bad stuff",
            },
            {
                'timestamp': "2014-03-21T06:24:26",
                'value': "Doing really bad stuff",
            }
        ],
        'status': "Ongoing",
        'related_ttps': {
            'scope': "exclusive",
            'ttps': [
                {
                    'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                    'ttp': {'title': "Stealth", 'version': '1.1.1'},
                }
            ]
        },
        'related_incidents': {
            'scope': "inclusive",
            'incidents': [
                {
                    'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                    'incident': {'idref': "example:Incident-2",
                                 'version': '1.1.1'},
                }
            ]
        },
        'related_indicators': {
            'scope': "inclusive",
            'indicators': [
                {
                    'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                    'indicator': {'idref': "example:Indicator-77",
                                  'version': '2.1.1'},
                }
            ]
        },
        'attribution': [{
            'scope': "inclusive",
            'threat_actors': [
                {
                    'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                    'threat_actor': {'title': "Campaign Actor #1",
                                     'version': '1.1.1'},
                },
                {
                    'threat_actor': {'idref': "example:ThreatActor-111",
                                     'version': '1.1.1'},
                },
            ],
        }],
        'associated_campaigns': {
            'scope': "inclusive",
            'campaigns': [
                {
                    'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                    'information_source': {'description': "Threat Feed"},
                    'campaign': {'title': "Baby Elephant", 'version': '1.1.1'},
                }
            ],
        },
        'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
        'activity': [
                {
                    'date_time': "2012-01-01T08:45:31",
                    'description': "The first bad thing"
                },
                {
                    'date_time': "2012-01-02T08:45:31",
                    'description': "Another bad thing"
                },
        ],
        'information_source': {
            'description': "A former member of the campaign.",
            'identity': {
                'name': "Mr. D. Fector",
            },
        },
        'handling': [
            {
                'marking_structures': [{
                    'marking_model_name': 'TLP',
                    'color': "RED",
                    'xsi:type': "tlpMarking:TLPMarkingStructureType",
                }]
            }
        ],
        'related_packages': {
            'packages': [
                {'idref': "example:Package-AB", 'relationship': "Parent"},
                {'idref': "example:Package-CD", 'relationship': "Child"}
            ]
        }
    }


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

"""Tests for various encoding issues throughout the library"""

import unittest
from StringIO import StringIO

import stix.bindings as bindings
from stix.core import STIXHeader, STIXPackage
from stix.campaign import Campaign
from stix.indicator import Indicator
from stix.incident import Incident
from stix.exploit_target import ExploitTarget
from stix.threat_actor import ThreatActor
from stix.ttp import TTP
from stix.common import StructuredText
import stix.incident.affected_asset as affected_asset

from stix.test import round_trip

UNICODE_STR = u"❤ ♎ ☀ ★ ☂ ♞ ☯ ☭ ☢ €☎⚑ ❄♫✂"

XML = \
u"""
<stix:STIX_Package
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:stix="http://stix.mitre.org/stix-1"
    xmlns:example="http://example.com/"
    id="example:Indicator-ba1d406e-937c-414f-9231-6e1dbe64fe8b" version="1.1.1" timestamp="2014-05-08T09:00:00.000000Z">
    <stix:STIX_Header>
        <stix:Title>{0}</stix:Title>
    </stix:STIX_Header>
</stix:STIX_Package>
""".format(UNICODE_STR)


class EncodingTests(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.orig_encoding = bindings.ExternalEncoding
        bindings.ExternalEncoding = 'utf-16'

    @classmethod
    def tearDownClass(cls):
        bindings.ExternalEncoding = cls.orig_encoding

    def _test_equal(self, obj1, obj2):
        self.assertEqual(obj1.title, obj2.title)
        self.assertEqual(obj1.description.value, obj2.description.value)
        self.assertEqual(obj1.short_description.value, obj2.short_description.value)

    def test_structured_text(self):
        s = StructuredText(UNICODE_STR)
        s2 = round_trip(s)
        self.assertEqual(s.value, s2.value)

    def test_asset_type(self):
        a = affected_asset.AssetType()
        a.count_affected = 1
        a2 = round_trip(a)
        self.assertEqual(a.count_affected, a2.count_affected)

    def test_stix_header(self):
        header = STIXHeader()
        header.title = UNICODE_STR
        header.description = UNICODE_STR
        header.short_description = UNICODE_STR
        header2 = round_trip(header)
        self._test_equal(header, header2)

    def test_indicator(self):
        i = Indicator()
        i.title = UNICODE_STR
        i.description = UNICODE_STR
        i.short_description = UNICODE_STR
        i2 = round_trip(i)
        self._test_equal(i, i2)

    def test_incident(self):
        i = Incident()
        i.title = UNICODE_STR
        i.description = UNICODE_STR
        i.short_description = UNICODE_STR
        i2 = round_trip(i)
        self._test_equal(i, i2)

    def test_ttp(self):
        t = TTP()
        t.title = UNICODE_STR
        t.description = UNICODE_STR
        t.short_description = UNICODE_STR
        t2 = round_trip(t)
        self._test_equal(t, t2)

    def test_ta(self):
        t = ThreatActor()
        t.title = UNICODE_STR
        t.description = UNICODE_STR
        t.short_description = UNICODE_STR
        t2 = round_trip(t)
        self._test_equal(t, t2)

    def test_et(self):
        e = ExploitTarget()
        e.title = UNICODE_STR
        e.description = UNICODE_STR
        e.short_description = UNICODE_STR
        e2 = round_trip(e)
        self._test_equal(e, e2)

    def test_campaign(self):
        c = Campaign()
        c.title = UNICODE_STR
        c.description = UNICODE_STR
        c.short_description = UNICODE_STR
        c2 = round_trip(c)
        self._test_equal(c, c2)

    def test_quote_xml(self):
        s = bindings.quote_xml(UNICODE_STR)
        self.assertEqual(s, UNICODE_STR)

    def test_quote_attrib(self):
        """Tests that the stix.bindings.quote_attrib method works properly
        on unicode inputs.

        Note:
            The quote_attrib method (more specifically, saxutils.quoteattr())
            adds quotation marks around the input data, so we need to strip
            the leading and trailing chars to test effectively
        """
        s = bindings.quote_attrib(UNICODE_STR)
        s = s[1:-1]
        self.assertEqual(s, UNICODE_STR)

    def test_quote_attrib_int(self):
        i = 65536
        s = bindings.quote_attrib(i)
        self.assertEqual(u'"65536"', s)

    def test_quote_attrib_bool(self):
        b = True
        s = bindings.quote_attrib(b)
        self.assertEqual(u'"True"', s)

    def test_quote_xml_int(self):
        i = 65536
        s = bindings.quote_xml(i)
        self.assertEqual(unicode(i), s)

    def test_quote_xml_bool(self):
        b = True
        s = bindings.quote_xml(b)
        self.assertEqual(unicode(b), s)

    def test_quote_xml_encoded(self):
        encoding = bindings.ExternalEncoding
        encoded = UNICODE_STR.encode(encoding)
        quoted = bindings.quote_xml(encoded)
        self.assertEqual(UNICODE_STR, quoted)

    def test_quote_attrib_encoded(self):
        encoding = bindings.ExternalEncoding
        encoded = UNICODE_STR.encode(encoding)
        quoted = bindings.quote_attrib(encoded)[1:-1]
        self.assertEqual(UNICODE_STR, quoted)

    def test_quote_xml_zero(self):
        i = 0
        s = bindings.quote_xml(i)
        self.assertEqual(unicode(i), s)

    def test_quote_attrib_zero(self):
        i = 0
        s = bindings.quote_attrib(i)
        self.assertEqual(u'"0"', s)

    def test_quote_xml_none(self):
        i = None
        s = bindings.quote_xml(i)
        self.assertEqual(u'', s)

    def test_quote_attrib_none(self):
        i = None
        s = bindings.quote_attrib(i)
        self.assertEqual(u'""', s)

    def test_quote_attrib_empty(self):
        i = ''
        s = bindings.quote_attrib(i)
        self.assertEqual(u'""', s)

    def test_quote_xml_empty(self):
        i = ''
        s = bindings.quote_xml(i)
        self.assertEqual(u'', s)

    def test_to_xml_utf16_encoded(self):
        encoding = 'utf-16'
        s = STIXHeader()
        s.title = UNICODE_STR
        xml = s.to_xml(encoding=encoding)
        self.assertTrue(UNICODE_STR in xml.decode(encoding))

    def test_to_xml_default_encoded(self):
        s = STIXHeader()
        s.title = UNICODE_STR
        xml = s.to_xml()
        self.assertTrue(UNICODE_STR in xml.decode('utf-8'))

    def test_to_xml_no_encoding(self):
        s = STIXHeader()
        s.title = UNICODE_STR
        xml = s.to_xml(encoding=None)
        self.assertTrue(isinstance(xml, unicode))
        self.assertTrue(UNICODE_STR in xml)

    def test_from_xml_utf16_encoded(self):
        utf16_xml = XML.encode('utf-16')
        sio = StringIO(utf16_xml)
        sp = STIXPackage.from_xml(sio, encoding='utf-16')
        header = sp.stix_header
        self.assertEqual(header.title, UNICODE_STR)

    def test_from_xml_default_encoded(self):
        utf8_xml = XML.encode('utf-8')
        sio = StringIO(utf8_xml)
        sp = STIXPackage.from_xml(sio)
        header = sp.stix_header
        self.assertEqual(header.title, UNICODE_STR)


    def test_utf16_roundtrip(self):
        sh = STIXHeader()
        sh.title = UNICODE_STR
        sp = STIXPackage()
        sp.stix_header = sh

        # serialize as utf-16
        xml16 = sp.to_xml(encoding='utf-16')

        # deserialize as utf-16
        sp2 = STIXPackage.from_xml(StringIO(xml16), encoding='utf-16')
        sh2 = sp2.stix_header

        # check that the titles align
        self.assertEqual(sh.title, sh2.title)


if __name__ == "__main__":
    unittest.main()



import unittest

from stix.test import EntityTestCase

from stix.threat_actor import ThreatActor


class ThreatActorTests(EntityTestCase, unittest.TestCase):
    klass = ThreatActor
    _full_dict = {
        'id': 'example:ThreatActor-1',
        'timestamp': "2014-01-31T06:14:46",
        'version': '1.1',
        'title': "BadGuy1",
        'description': "This is a long description about a threat actor.",
        'short_description': "A bad guy",
        'identity': {
            'name': "Mr. Evil",
        },
        'types': [
            {'value': {"value" : "Hacker", 
                       "xsi:type" : "stixVocabs:ThreatActorTypeVocab-1.0"}},
        ],
        'motivations': [
            {'value': {"value" : "Ego", 
                       "xsi:type" : "stixVocabs:MotivationVocab-1.1"}},
            {'value': {"value" : "Opportunistic", 
                       "xsi:type" : "stixVocabs:MotivationVocab-1.1"}},
        ],
        'sophistications': [
            {'value': {"value" : "Novice", 
                       "xsi:type" : "stixVocabs:ThreatActorSophisticationVocab-1.0"}},
        ],
        'intended_effects': [
            {'value': {"value" : "Destruction", 
                       "xsi:type" : "stixVocabs:IntendedEffectVocab-1.0"}},
            {'value': {"value" : "Disruption", 
                       "xsi:type" : "stixVocabs:IntendedEffectVocab-1.0"}},
        ],
        'planning_and_operational_supports': [
            {'value': {"value" : "Data Exploitation", 
                       "xsi:type" : 'stixVocabs:PlanningAndOperationalSupportVocab-1.0.1'}},
        ],
        'observed_ttps': {
            'scope': "exclusive",
            'ttps': [
                {
                    'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                    'ttp': {'title': "A TTP", 'version': '1.1'},
                }
            ],
        },
        'associated_campaigns': {
            'scope': "inclusive",
            'campaigns': [
                {
                    'information_source': {'description': "Mailing List"},
                    'campaign': {'title': "Campaign1", 'version': '1.1'},
                }
            ],
        },
        'associated_actors': {
            'scope': "inclusive",
            'threat_actors': [
                {
                    'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                    'threat_actor': {'title': "Related Bad Guy!", 'version': '1.1'},
                }
            ],
        },
        'handling': [
            {
                'marking_structures': [{
                    'marking_model_name': 'TLP',
                    'color': "GREEN",
                    'xsi:type': "tlpMarking:TLPMarkingStructureType",
                }]
            }
        ],
        'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
        'information_source': {
            'description': "Mr. Evil's enemy",
            'identity': {
                'name': "Ms. Good",
            },
        },
        'related_packages': {
            'packages': [
                {'idref': "example:Package-1234", 'relationship': "Parent"},
                {'idref': "example:Package-1235", 'relationship': "Child"}
            ]
        }
    }


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

'''
Description: Round-trip example. This script takes a STIX instance document from XML to
a binding object, then to a api object and then to a dictionary. That dictionary is then
converted back into an api object, which is then used to generate an XML document.
'''
import io
from pprint import pprint

from stix.core import STIXPackage
from stix.indicator import Indicator
import stix.bindings.stix_core as stix_core_binding

def main():
    fn = 'sample.xml'
    stix_package = STIXPackage.from_xml(fn)
    stix_dict = stix_package.to_dict() # parse to dictionary
    pprint(stix_dict)

    stix_package_two = STIXPackage.from_dict(stix_dict) # create python-stix object from dictionary
    xml = stix_package_two.to_xml() # generate xml from python-stix object
    print(xml)

if __name__ == '__main__':
    main()



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.utils
import stix.bindings.incident as incident_binding
from stix.common import DateTimeWithPrecision
from .contributors import Contributors, Contributor
from stix.coa import CourseOfAction

class COATaken(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding = incident_binding
    _binding_class = _binding.COATakenType
    
    def __init__(self, course_of_action=None):
        self.time = None
        self.course_of_action = course_of_action
        self.contributors = Contributors()
        
    def add_contributor(self, value):
        if not value:
            return
        elif isinstance(value, Contributor):
            self.contributors.append(value)
        else:
            raise ValueError("Cannot add type %s to contributors" % type(value))
    
    @property
    def time(self):
        return self._time
    
    @time.setter
    def time(self, value):
        if not value:
            self._time = None
        elif isinstance(value, COATime):
            self._time = value
        else:
            raise ValueError("Cannot set time to type %s" % type(value))
    
    @property
    def course_of_action(self):
        return self._course_of_action
    
    @course_of_action.setter
    def course_of_action(self, value):
        if not value:
            self._course_of_action = None
        elif isinstance(value, CourseOfAction):
            self._course_of_action = value
        else:
            raise ValueError("Cannot set course_of_action to type %s" % type(value))
    
    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()
            
        return_obj.time = COATime.from_obj(obj.Time)
        return_obj.contributors = Contributors.from_obj(obj.Contributors)
        return_obj.course_of_action = CourseOfAction.from_obj(obj.Course_Of_Action)
        
        return return_obj

    def to_obj(self, return_obj=None, ns_info=None):
        super(COATaken, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()
            
        if self.time:
            return_obj.Time = self.time.to_obj(ns_info=ns_info)
        if self.contributors:
            return_obj.Contributors = self.contributors.to_obj(ns_info=ns_info)
        if self.course_of_action:
            return_obj.Course_Of_Action = self.course_of_action.to_obj(ns_info=ns_info)
        
        return return_obj
    
    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        if not return_obj:
            return_obj = cls()
            
        return_obj.time = COATime.from_dict(d.get('time'))
        return_obj.contributors = Contributors.from_dict(d.get('contributors'))
        return_obj.course_of_action = CourseOfAction.from_dict(d.get('course_of_action'))
        
        return return_obj
    
    def to_dict(self):
        d = {}

        if self.time:
            d['time'] = self.time.to_dict()
        if self.contributors:
            d['contributors'] = self.contributors.to_dict()
        if self.course_of_action:
            d['course_of_action'] = self.course_of_action.to_dict()

        return d
    
    
class COATime(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding = incident_binding
    _binding_class = _binding.COATimeType
    
    def __init__(self, start=None, end=None):
        self.start = start
        self.end = end
        
    @property
    def start(self):
        return self._start
    
    @start.setter
    def start(self, value):
        if not value:
            self._start = None
        elif isinstance(value, DateTimeWithPrecision):
            self._start = value
        else:
            self._start = DateTimeWithPrecision(value)
    
    @property
    def end(self):
        return self._end
    
    @end.setter
    def end(self, value):
        if not value:
            self._end = None
        elif isinstance(value, DateTimeWithPrecision):
            self._end = value
        else:
            self._end = DateTimeWithPrecision(value)
    
    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()
            
        return_obj.start = DateTimeWithPrecision.from_obj(obj.Start)
        return_obj.end = DateTimeWithPrecision.from_obj(obj.End)
        return return_obj
    
    def to_obj(self, return_obj=None, ns_info=None):
        super(COATime, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()
            
        if self.start:
            return_obj.Start = self.start.to_obj(ns_info=ns_info)
        if self.end:
            return_obj.End = self.end.to_obj(ns_info=ns_info)
        
        return return_obj
    
    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        if not return_obj:
            return_obj = cls()
            
        return_obj.start = DateTimeWithPrecision.from_dict(d.get('start'))
        return_obj.end = DateTimeWithPrecision.from_dict(d.get('end'))
        return return_obj
    
    def to_dict(self):
        d = {}
        
        if self.start:
            d['start'] = self.start.to_dict()
        if self.end:
            d['end'] = self.end.to_dict()
        
        return d
